function CGame(oInfo){
    var _bFirstTap = true;
    var _iCurState = -1;
    var _iScore;
    var _iBestScore;
    var _iTimeElaps;
    var _oBg;
    var _oHero;
    var _oScrollingBg;
    var _oObstacleController;
    
    var _oInterface;
    var _oEndPanel;
    
    this._init = function(){
        _iScore = 0;
        _iBestScore = 0;
        
        _oBg = new createjs.Bitmap(s_oSpriteLibrary.getSprite('bg_game'));
        s_oStage.addChild(_oBg);
	
        _oObstacleController = new CObstacle(s_oSpriteLibrary.getSprite('obstacle'));
        _oScrollingBg = new CScrollingBg(s_oSpriteLibrary.getSprite('bg_scroll'));
        
        _oHero = new CHero(HERO_START_X,HERO_START_Y,s_oSpriteLibrary.getSprite('hero'));

        _oInterface = new CInterface();
        
        
        _iCurState = GAME_STATE_BEFORE_START;
    };
    
    this.unload = function(){
        s_oStage.removeChild(_oBg);
        _oScrollingBg.unload();
        _oObstacleController.unload();
        _oHero.unload();
        
        if(_oEndPanel){
            _oEndPanel.unload();
        } 
        
        _oInterface.unload();
    };

    this.tapScreen = function(){
        if(_iCurState === GAME_STATE_GAME_OVER){
            return;
        }

        createjs.Sound.play("tap");
        if(_bFirstTap){
            _bFirstTap = false;
            _oHero.start();
            _iTimeElaps = 0;
            _iCurState = GAME_STATE_TRAINING;//GAME_STATE_START;
        }else{
            _oHero.moveUp();
        }
    };
    
    this._resetLevel = function(){
        _iScore =0;
        _oInterface.refreshScore(_iScore);
        
        _oHero.reset();
        _oObstacleController.restartLevel();
        _bFirstTap = true;

        _iCurState = GAME_STATE_BEFORE_START;
    };
    
    this.collisionFound = function(){
        _iCurState = GAME_STATE_GAME_OVER;
    };
    
    this.increaseScore = function(){
        _iScore++;
        _oInterface.refreshScore(_iScore);
    };
    
    this._gameOver = function(){  
        _iCurState = -1;
        
        if(_iScore > _iBestScore){
            _iBestScore = _iScore; 
        }

        _oEndPanel = CEndPanel(s_oSpriteLibrary.getSprite('msg_box'));
        _oEndPanel.show(_iScore,_iBestScore);
    };
    
    this.onRestartGame = function(){
        this._resetLevel();
    };

    this.onExit = function(){
        this.unload();
        s_oMain.gotoMenu();
		
	$(s_oMain).trigger("restart");
    };
    
    this.update = function(){
        switch(_iCurState){
            case GAME_STATE_BEFORE_START:{
                _oScrollingBg.update(BG_SPEED);
                break;
            }
            
            case GAME_STATE_TRAINING:{
                _iTimeElaps += s_iTimeElaps;
                if(_iTimeElaps>TIME_TRAINING){
                    _iTimeElaps = 0;
                    _iCurState = GAME_STATE_START;
                }else{
                    _oHero.update((CANVAS_HEIGHT - _oScrollingBg.getHeight()));
                    _oScrollingBg.update(BG_SPEED);
                }
                break;
            }
            
            case GAME_STATE_START:{
                _oHero.update((CANVAS_HEIGHT - _oScrollingBg.getHeight()));
                _oScrollingBg.update(BG_SPEED);
                _oObstacleController.update(_oHero.getSprite(),BG_SPEED);
                break;
            }
            
            case GAME_STATE_GAME_OVER:{
                _oHero.update((CANVAS_HEIGHT - _oScrollingBg.getHeight()));
                break;
            }
        }
    };

    s_oGame=this;

    HERO_START_X = oInfo.hero_x;
    HERO_START_Y = oInfo.hero_y;
    HERO_DOWN_ACCELLERATION = oInfo.hero_down_speed;
    BG_SPEED = oInfo.bg_speed;
    TIME_TRAINING = oInfo.time_training;
    DIST_AMONG_OBSTACLES = oInfo.dist_among_obst;
    OBSTACLE_HEIGHT_DIST = oInfo.obst_height_dist;

    this._init();
}

var s_oGame;